@@ -7,13 +7,13 @@ var ExtCommonUtils = {
 
 	content      : 'chrome://ctxextensions/content/',
 	locale       : 'chrome://ctxextensions/locale/',
-	
+	 
 	// vpeB 
 	// properties
-	
-	get browserURI() 
+	 
+	get mainURI() 
 	{
-		if (!this._browserURI) {
+		if (this._mainURI === void(0)) {
 			var uri = this.getPref('browser.chromeURL');
 			if (!uri) {
 				try {
@@ -23,18 +23,26 @@ var ExtCommonUtils = {
 				catch(e) {
 				}
 			}
-			if (uri.charAt(uri.length-1) == '/')
-				uri = uri.replace(/chrome:\/\/([^\/]+)\/content\//, 'chrome://$1/content/$1.xul');
-			this._browserURI = uri;
+			if (uri) {
+				if (uri.charAt(uri.length-1) == '/')
+					uri = uri.replace(/chrome:\/\/([^\/]+)\/content\//, 'chrome://$1/content/$1.xul');
+			}
+			this._mainURI = uri;
 		}
-		return this._browserURI;
+		return this._mainURI;
 	},
-	_browserURI : null,
+//	_mainURI : null,
   
+	get isBrowser() 
+	{
+		return (this.mainURI && this.mainURI.indexOf('chrome://browser/') > -1) ? true : false ;
+	},
+ 
 	get contextMenu() 
 	{
 		if (!this.mContextMenu)
-			this.mContextMenu = document.getElementById('contentAreaContextMenu');
+			this.mContextMenu = document.getElementById('contentAreaContextMenu') || // Firefox
+								document.getElementById('messagePaneContext'); // Thunderbird
 		return this.mContextMenu;
 	},
 	mContextMenu : null,
@@ -42,7 +50,8 @@ var ExtCommonUtils = {
 	get browser() 
 	{
 		if (this._browser === void(0)) {
-			this._browser = document.getElementById('content');
+			this._browser = document.getElementById('content') || // Firefox
+							document.getElementById('messagepane'); // Thunderbird
 			if (this._browser &&
 				(
 					this._browser.localName != 'browser' &&
@@ -82,14 +91,21 @@ var ExtCommonUtils = {
 		return document.getElementsByTagNameNS('http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul', 'tabbrowser');
 	},
   
-	get browserWindow() 
+	get mainWindow() 
 	{
-		return this.getTopWindowOf('navigator:browser');
+		return this.isBrowser ?
+				this.getTopWindowOf('navigator:browser') : // Firefox
+				( // Thunderbird
+					this.getTopWindowOf('mail:3pane') ||
+					this.getTopWindowOf('mail:messageWindow')
+				);
 	},
  
-	get browserWindows() 
+	get mainWindows() 
 	{
-		return this.getWindowsOf('navigator:browser');
+		return this.isBrowser ?
+				this.getWindowsOf('navigator:browser') :
+				this.getWindowsOf('mail:3pane').concat(this.getWindowsOf('mail:messageWindow'));
 	},
  
 	get datasource() 
@@ -629,9 +645,7 @@ var ExtCommonUtils = {
 	// URLt@CpXɕϊ 
 	getFileFromURLSpec : function(aURI)
 	{
-		var URI = this.IOService.newURI(aURI, null, null);
-
-		if (!URI.schemeIs('file')) return '';
+		if ((aURI || '').indexOf('file://') != 0) return '';
 
 		var fileHandler = this.IOService.getProtocolHandler('file').QueryInterface(Components.interfaces.nsIFileProtocolHandler);
 		return fileHandler.getFileFromURLSpec(aURI);
@@ -705,7 +719,14 @@ var ExtCommonUtils = {
 			dir[i] = this.getParentDir(dir[i-1]);
 		return dir;
 	},
-    
+   
+	openURIInExternalApp : function(aURI) 
+	{
+		var uri = Components.classes['@mozilla.org/network/io-service;1'].getService(Components.interfaces.nsIIOService).newURI(aURI, null, null);
+		var protocolSvc = Components.classes['@mozilla.org/uriloader/external-protocol-service;1'].getService(Components.interfaces.nsIExternalProtocolService);
+		protocolSvc.loadUrl(uri);
+	},
+  
 	// t@C 
 	
 	// nꂽpXnsIFilẽIuWFNg𐶐 
@@ -1124,7 +1145,7 @@ var ExtCommonUtils = {
 	},
    
 	// DOM 
-	
+	 
 	getTopWindowOf : function(aType) 
 	{
 		return this.WINMAN.getMostRecentWindow(aType);
@@ -1496,11 +1517,13 @@ var ExtCommonUtils = {
 	// nsIXULTemplateBuilder.rebuild() ̑ 
 	rebuildFromTemplate : function(aContainer)
 	{
+		if (!aContainer) return;
+
 		var i, j;
 		const XULNS = 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul';
 
 		var template = aContainer.hasAttribute('template') ? document.getElementById(aContainer.getAttribute('template')) : aContainer.getElementsByTagNameNS(XULNS, 'template')[0] ;
-		if (!template.hasAttribute('ext-template')) return;
+		if (!template || !template.hasAttribute('ext-template')) return;
 
 		if (!('extTemplate' in template))
 			eval('template.extTemplate = '+template.getAttribute('ext-template'));
@@ -1531,7 +1554,18 @@ var ExtCommonUtils = {
 		}
 //dump(aContainer.getAttribute('ext-datasource')+' / '+obj.length+'('+aContainer.childNodes.length+')\n');
 	},
- 
+	 
+	cleanUpInvalidKeys : function() 
+	{
+		var nodes = this.getNodesFromXPath('/descendant::*[local-name() = "key" and (@key = "" or not(@key)) and (@keycode = "" or not(@keycode))]');
+		var node;
+		for (var i = nodes.snapshotLength-1; i > -1; i--)
+		{
+			node = nodes.snapshotItem(i);
+			node.parentNode.removeChild(node);
+		}
+	},
+ 	 
 	getNodesFromXPath : function(aXPath, aContextNode, aType) 
 	{
 		// http://www.hawk.34sp.com/stdpls/xml/
@@ -1820,6 +1854,7 @@ var ExtCommonUtils = {
 				this.rebuildFromTemplate(nodes[i]);
 			}
 		}
+		this.cleanUpInvalidKeys();
 	},
 	loadPrefs : function()
 	{
