@@ -109,7 +109,7 @@ var ExtFunc = {
 		var info = this.service.contentInfo();
 		this.service.scrollTo(info.headings[i].node);
 		info.headingsCurrentIndex = i;
-		aEvent.preventBubble();
+		aEvent.stopPropagation();
 		return;
 	},
  
@@ -118,7 +118,7 @@ var ExtFunc = {
 	{
 		var uri = aEvent.target.value;
 		this.service.loadURI(uri, this.utils.makeURIFromSpec(this.service.currentURI()), this.utils.getPref('ctxextensions.showResultIn.navigations'));
-		aEvent.preventBubble();
+		aEvent.stopPropagation();
 	},
  
 	// s̕\ 
@@ -349,8 +349,8 @@ var ExtFunc = {
 	ApplyStyle : function(aEventOrID, aShouldSave, aWindow)
 	{
 		if (typeof aEventOrID != 'string' &&
-			'preventBubble' in aEventOrID) {
-			aEventOrID.preventBubble();
+			'stopPropagation' in aEventOrID) {
+			aEventOrID.stopPropagation();
 			if (aEventOrID.altKey || aEventOrID.ctrlKey || aEventOrID.metaKey)
 				return this.service.editRDFItem(aEventOrID, 'StyleSheets');
 		}
@@ -515,8 +515,8 @@ var ExtFunc = {
 
 		if (aEventOrID &&
 			typeof aEventOrID != 'string' &&
-			'preventBubble' in aEventOrID) {
-			aEventOrID.preventBubble();
+			'stopPropagation' in aEventOrID) {
+			aEventOrID.stopPropagation();
 			name        = aEventOrID.target.label;
 			embedded[0] = aEventOrID.target.getAttribute('embeddedSheet');
 			value[0]    = embedded[0] ? this.utils.unescape(aEventOrID.target.getAttribute('embeddedSheet')) : aEventOrID.target.value ;
@@ -585,8 +585,8 @@ var ExtFunc = {
 	SendStr : function(aEventOrID, aString)
 	{
 		if (typeof aEventOrID != 'string' &&
-			'preventBubble' in aEventOrID) {
-			aEventOrID.preventBubble();
+			'stopPropagation' in aEventOrID) {
+			aEventOrID.stopPropagation();
 			if (aEventOrID.altKey || aEventOrID.ctrlKey || aEventOrID.metaKey)
 				return this.service.editRDFItem(aEventOrID, 'SendStr');
 		}
@@ -654,8 +654,8 @@ var ExtFunc = {
 	SendURI : function(aEventOrID, aShouldOpenTopFrame, aURI)
 	{
 		if (typeof aEventOrID != 'string' &&
-			'preventBubble' in aEventOrID) {
-			aEventOrID.preventBubble();
+			'stopPropagation' in aEventOrID) {
+			aEventOrID.stopPropagation();
 			if (aEventOrID.altKey || aEventOrID.ctrlKey || aEventOrID.metaKey)
 				return this.service.editRDFItem(aEventOrID, 'SendURI');
 		}
@@ -678,12 +678,12 @@ var ExtFunc = {
 				this.service.loadURI(uri, null, this.service.NEW_WINDOW);
 				break;
 			case 'NewTab':
-				if (this.utils.browserWindow.ExtService.currentURI(true) != 'about:blank') {
+				if (this.utils.mainWindow.ExtService.currentURI(true) != 'about:blank') {
 					this.service.openNewTab(uri, null, this.service.NEW_TAB);
 					break;
 				}
 			case 'NewBackgroundTab':
-				if (this.utils.browserWindow.ExtService.currentURI(true) != 'about:blank') {
+				if (this.utils.mainWindow.ExtService.currentURI(true) != 'about:blank') {
 					this.service.openNewTab(uri, null, this.service.NEW_BG_TAB);
 					break;
 				}
@@ -700,8 +700,8 @@ var ExtFunc = {
 	ExecApps : function(aEventOrID, aShouldOpenTopFrame, aURI, aString)
 	{
 		if (typeof aEventOrID != 'string' &&
-			'preventBubble' in aEventOrID) {
-			aEventOrID.preventBubble();
+			'stopPropagation' in aEventOrID) {
+			aEventOrID.stopPropagation();
 			if (aEventOrID.altKey || aEventOrID.ctrlKey || aEventOrID.metaKey)
 				return this.service.editRDFItem(aEventOrID, 'ExecApps');
 		}
@@ -743,7 +743,7 @@ var ExtFunc = {
 	CustomScripts : function(aEventOrID, aScript, aWindow)
 	{
 		if (aEventOrID && typeof aEventOrID == 'object') { //event
-			aEventOrID.preventBubble();
+			aEventOrID.stopPropagation();
 			if (aEventOrID.altKey || aEventOrID.ctrlKey || aEventOrID.metaKey)
 				return this.service.editRDFItem(aEventOrID, 'CustomScripts');
 		}
@@ -785,10 +785,52 @@ var ExtFunc = {
 
 		try {
 			// windowIuWFNg̃\bhɂ
-			eval(['window.extCustomScripts[id] = function() { with(window) {', script, '} }'].join('\n\r'));
+			eval([
+				'window.extCustomScripts[id] = function() { with (window) {',
+					'var _window = window._window;',
+					'var _contextualURI = window._contextualURI;',
+					'var _selection = window._selection;',
+					'var _selectionSource = window._selectionSource;',
+					'var _selectionSourceXML = window._selectionSourceXML;',
+					'var _selectionNodes = window._selectionNodes;',
+					'var _focusedElement = window._focusedElement;',
+
+					// ȑÕo[W'
+					'var _getSelection = window._getSelection;',
+					'var _getSelectionSource = window._getSelectionSource;',
+					'var _getSelectionNodes = window._getSelectionNodes;',
+					'var _isOnline = window._isOnline;',
+					'var _inFrame = window._inFrame;',
+					'var _popupNode = window._popupNode;',
+					'var _profileURI = window._profileURI;',
+					'var _installedURI = window._installedURI;',
+					'var _temporaryURI = window._temporaryURI;',
+					'var _homeURI = window._homeURI;',
+					'var _profilePath = window._profilePath;',
+					'var _installedPath = window._installedPath;',
+					'var _temporaryPath = window._temporaryPath;',
+					'var _homePath = window._homePath;',
+
+					script,
+				' } }'
+			].join('\n\r'));
+			return window.extCustomScripts[id]();
+		}
+		catch(e) { // Fx 3ł̓G[ɂȂiwithĂHj
+			try {
+				eval([
+					'window.extCustomScripts[id] = function() { with (window) {',
+						script,
+					' } }'
+				].join('\n\r'));
+			}
+			catch(e) {
+				alert(e);
+				return false;
+			}
+		}
+		try {
 			return window.extCustomScripts[id]();
-//			delete window.extCustomScripts[id];
-//			return ret;
 		}
 		catch(e) {
 			alert(e);
